from __future__ import unicode_literals

import pprint

from django.utils.module_loading import import_string

import requests

from .conf import app_settings


class SlackException(ValueError):
    pass


def console(data, url):
    print("-" * 79)
    print("Slack message:")
    pprint.pprint(data, indent=4)
    print("-" * 79)


def handle_error(resp):
    if resp.headers['Content-Type'].startswith('application/json'):
        result = resp.json()

        if not result['ok']:
            raise SlackException(result['error'])

        return

    if resp.text != 'ok':
        raise SlackException(resp.text)


def default(data, url):
    resp = requests.post(url, data=data)
    handle_error(resp)


def get_backend():
    """
    Wrap the backend in a function to not load it at import time.
    get_backend() caches the backend on first call.
    """
    if get_backend.backend is None:
        get_backend.backend = import_string(app_settings.BACKEND)
    return get_backend.backend
get_backend.backend = None



from django.core.exceptions import ImproperlyConfigured

from .backends import get_backend
from .conf import app_settings


if app_settings.TOKEN is None:
    raise ImproperlyConfigured('CHANNELS_SLACK_TOKEN is required.')


def handle_notification(message):
    backend = get_backend()
    endpoint_url = app_settings.ENDPOINT_URL

    slack_data = dict(
        token = app_settings.TOKEN,
        channel = app_settings.CHANNEL,
        icon_emoji = app_settings.ICON_EMOJI,
        username = app_settings.USERNAME,
        as_user = False,
    )

    slack_data.update(message.data)

    backend(slack_data, endpoint_url)



from django.conf import settings


def setting(suffix, default):
    @property
    def fn(self):
        return getattr(settings, 'CHANNELS_SLACK_%s' % suffix, default)
    return fn


class AppSettings(object):
    ENDPOINT_URL = 'https://slack.com/api/chat.postMessage'
    DEFAULT_BACKEND = 'channels_slack_notifications.backends.console'

    TOKEN = setting('TOKEN', None)
    CHANNEL = setting('CHANNEL', '#general')
    USERNAME = setting('USERNAME', 'notifcationbot')
    ICON_EMOJI = setting('ICON_EMOJI', ':robot_face:')

    BACKEND = setting('BACKEND', DEFAULT_BACKEND)

app_settings = AppSettings()



from setuptools import setup, find_packages


setup(
    name="channels-slack-notifications",
    version='0.0.1',
    description='Send notification to slack using channels',
    keywords="django, channels, slack",
    author="Avocado Co-op <dev@avocadoproject.ca>",
    author_email="dev@avocadoproject.ca",
    url="https://github.com/AvocadoCoop/channels-slack-notifications",
    license="MIT License",
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
        "Environment :: Web Environment",
    ],
)



from django.core.management.base import BaseCommand

from channels_slack_notifications.backends import get_backend
from channels_slack_notifications.conf import app_settings


class Command(BaseCommand):
    help = 'Sends a test "Hello, Slack!" message to test your slack configuration'

    def handle(self, *args, **options):
        message = "Hello, Slack!"
        backend = get_backend()
        endpoint_url = app_settings.ENDPOINT_URL

        slack_data = dict(
            text = message,
            token = app_settings.TOKEN,
            channel = app_settings.CHANNEL,
            icon_emoji = app_settings.ICON_EMOJI,
            username = app_settings.USERNAME,
        )

        backend(slack_data, endpoint_url)

        print('message sent')
